@@ -5,10 +5,8 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 #endif
 {-# OPTIONS_GHC -Wall #-}
+#include "free-common.h"
 
-#ifndef MIN_VERSION_base
-#define MIN_VERSION_base(x,y,z) 1
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Applicative.Free
@@ -34,6 +32,7 @@ module Control.Applicative.Free
   , runAp
   , runAp_
   , liftAp
+  , iterAp
   , hoistAp
   , retractAp
 
@@ -42,6 +41,7 @@ module Control.Applicative.Free
   ) where
 
 import Control.Applicative
+import Control.Comonad (Comonad(..))
 import Data.Functor.Apply
 import Data.Typeable
 
@@ -88,11 +88,23 @@ instance Applicative (Ap f) where
   Pure f <*> y = fmap f y
   Ap x y <*> z = Ap x (flip <$> y <*> z)
 
+instance Comonad f => Comonad (Ap f) where
+  extract (Pure a) = a
+  extract (Ap x y) = extract y (extract x)
+  duplicate (Pure a) = Pure (Pure a)
+  duplicate (Ap x y) = Ap (duplicate x) (extend (flip Ap) y)
+  
 -- | A version of 'lift' that can be used with just a 'Functor' for @f@.
 liftAp :: f a -> Ap f a
 liftAp x = Ap x (Pure id)
 {-# INLINE liftAp #-}
 
+-- | Tear down a free 'Applicative' using iteration.
+iterAp :: Functor g => (g a -> a) -> Ap g a -> a
+iterAp algebra = go
+  where go (Pure a) = a
+        go (Ap underlying apply) = algebra (go . (apply <*>) . pure <$> underlying)
+
 -- | Given a natural transformation from @f@ to @g@ this gives a monoidal natural transformation from @Ap f@ to @Ap g@.
 hoistAp :: (forall a. f a -> g a) -> Ap f b -> Ap g b
 hoistAp _ (Pure a) = Pure a
