@@ -307,10 +307,12 @@ instance (Applicative f, Applicative m, Monad m) => Monad (FreeT f m) where
   FreeT m >>= f = FreeT $ m >>= \v -> case v of
     Pure a -> runFreeT (f a)
     Free w -> return (Free (fmap (>>= f) w))
-  fail = Fail.fail
-
-instance (Applicative f, Applicative m, Monad m) => Fail.MonadFail (FreeT f m) where
+#if !MIN_VERSION_base(4,13,0)
   fail e = FreeT (fail e)
+#endif
+
+instance (Applicative f, Applicative m, Fail.MonadFail m) => Fail.MonadFail (FreeT f m) where
+  fail e = FreeT (Fail.fail e)
 
 instance MonadTrans (FreeT f) where
   lift = FreeT . liftM Pure
